@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2018 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
@@ -42,8 +42,11 @@
 #include <QtCore/QCoreApplication>
 #include <QtCore/QVariant>
 #include <QtCore/QSharedData>
+#if QT_CONFIG(settings)
 #include <QtCore/QSettings>
+#endif
 #include <QtCore/QUrl>
+#include <QtCore/QVector>
 #include <QtGui/QColor>
 
 #include <algorithm>
@@ -283,7 +286,7 @@ QColorDialogStaticData::QColorDialogStaticData() : customSet(false)
 
 void QColorDialogStaticData::readSettings()
 {
-#ifndef QT_NO_SETTINGS
+#if QT_CONFIG(settings)
     const QSettings settings(QSettings::UserScope, QStringLiteral("QtProject"));
     for (int i = 0; i < int(CustomColorCount); ++i) {
         const QVariant v = settings.value(QLatin1String("Qt/customColors/") + QString::number(i));
@@ -295,8 +298,9 @@ void QColorDialogStaticData::readSettings()
 
 void QColorDialogStaticData::writeSettings() const
 {
-#ifndef QT_NO_SETTINGS
+#if QT_CONFIG(settings)
     if (customSet) {
+        const_cast<QColorDialogStaticData*>(this)->customSet = false;
         QSettings settings(QSettings::UserScope, QStringLiteral("QtProject"));
         for (int i = 0; i < int(CustomColorCount); ++i)
             settings.setValue(QLatin1String("Qt/customColors/") + QString::number(i), customRgb[i]);
@@ -663,18 +667,18 @@ QStringList QFileDialogOptions::history() const
 
 void QFileDialogOptions::setLabelText(QFileDialogOptions::DialogLabel label, const QString &text)
 {
-    if (label >= 0 && label < DialogLabelCount)
+    if (unsigned(label) < unsigned(DialogLabelCount))
         d->labels[label] = text;
 }
 
 QString QFileDialogOptions::labelText(QFileDialogOptions::DialogLabel label) const
 {
-    return (label >= 0 && label < DialogLabelCount) ? d->labels[label] : QString();
+    return (unsigned(label) < unsigned(DialogLabelCount)) ? d->labels[label] : QString();
 }
 
 bool QFileDialogOptions::isLabelExplicitlySet(DialogLabel label)
 {
-    return label >= 0 && label < DialogLabelCount && !d->labels[label].isEmpty();
+    return unsigned(label) < unsigned(DialogLabelCount) && !d->labels[label].isEmpty();
 }
 
 QUrl QFileDialogOptions::initialDirectory() const
@@ -785,7 +789,8 @@ class QMessageDialogOptionsPrivate : public QSharedData
 public:
     QMessageDialogOptionsPrivate() :
         icon(QMessageDialogOptions::NoIcon),
-        buttons(QPlatformDialogHelper::Ok)
+        buttons(QPlatformDialogHelper::Ok),
+        nextCustomButtonId(QPlatformDialogHelper::LastButton + 1)
     {}
 
     QString windowTitle;
@@ -794,6 +799,8 @@ public:
     QString informativeText;
     QString detailedText;
     QPlatformDialogHelper::StandardButtons buttons;
+    QVector<QMessageDialogOptions::CustomButton> customButtons;
+    int nextCustomButtonId;
 };
 
 QMessageDialogOptions::QMessageDialogOptions(QMessageDialogOptionsPrivate *dd)
@@ -885,6 +892,35 @@ QPlatformDialogHelper::StandardButtons QMessageDialogOptions::standardButtons()
     return d->buttons;
 }
 
+int QMessageDialogOptions::addButton(const QString &label, QPlatformDialogHelper::ButtonRole role,
+                                     void *buttonImpl)
+{
+    const CustomButton b(d->nextCustomButtonId++, label, role, buttonImpl);
+    d->customButtons.append(b);
+    return b.id;
+}
+
+static inline bool operator==(const QMessageDialogOptions::CustomButton &a,
+                              const QMessageDialogOptions::CustomButton &b) {
+    return a.id == b.id;
+}
+
+void QMessageDialogOptions::removeButton(int id)
+{
+    d->customButtons.removeOne(CustomButton(id));
+}
+
+const QVector<QMessageDialogOptions::CustomButton> &QMessageDialogOptions::customButtons()
+{
+    return d->customButtons;
+}
+
+const QMessageDialogOptions::CustomButton *QMessageDialogOptions::customButton(int id)
+{
+    int i = d->customButtons.indexOf(CustomButton(id));
+    return (i < 0 ? nullptr : &d->customButtons.at(i));
+}
+
 QPlatformDialogHelper::ButtonRole QPlatformDialogHelper::buttonRole(QPlatformDialogHelper::StandardButton button)
 {
     switch (button) {
