@@ -161,11 +161,10 @@ Qt::Orientation QSplitterHandle::orientation() const
 
 
 /*!
-    Returns \c true if widgets are resized dynamically (opaquely), otherwise
-    returns \c false. This value is controlled by the QSplitter.
+    Returns \c true if widgets are resized dynamically (opaquely) while interactively moving the
+    splitter. Otherwise returns \c false. This value is controlled by the QSplitter.
 
     \sa QSplitter::opaqueResize()
-
 */
 bool QSplitterHandle::opaqueResize() const
 {
@@ -826,7 +825,7 @@ QSplitterLayoutStruct *QSplitterPrivate::findWidget(QWidget *w) const
         if (list.at(i)->widget == w)
             return list.at(i);
     }
-    return 0;
+    return nullptr;
 }
 
 
@@ -855,7 +854,7 @@ void QSplitterPrivate::insertWidget_helper(int index, QWidget *widget, bool show
 QSplitterLayoutStruct *QSplitterPrivate::insertWidget(int index, QWidget *w)
 {
     Q_Q(QSplitter);
-    QSplitterLayoutStruct *sls = 0;
+    QSplitterLayoutStruct *sls = nullptr;
     int i;
     int last = list.count();
     for (i = 0; i < list.size(); ++i) {
@@ -872,12 +871,9 @@ QSplitterLayoutStruct *QSplitterPrivate::insertWidget(int index, QWidget *w)
     if (sls) {
         list.move(i,index);
     } else {
-        QSplitterHandle *newHandle = 0;
         sls = new QSplitterLayoutStruct;
-        QString tmp = QLatin1String("qt_splithandle_");
-        tmp += w->objectName();
-        newHandle = q->createHandle();
-        newHandle->setObjectName(tmp);
+        QSplitterHandle *newHandle = q->createHandle();
+        newHandle->setObjectName(QLatin1String("qt_splithandle_") + w->objectName());
         sls->handle = newHandle;
         sls->widget = w;
         w->lower();
@@ -1248,7 +1244,7 @@ QSplitterHandle *QSplitter::handle(int index) const
 {
     Q_D(const QSplitter);
     if (index < 0 || index >= d->list.size())
-        return 0;
+        return nullptr;
     return d->list.at(index)->handle;
 }
 
@@ -1262,7 +1258,7 @@ QWidget *QSplitter::widget(int index) const
 {
     Q_D(const QSplitter);
     if (index < 0 || index >= d->list.size())
-        return 0;
+        return nullptr;
     return d->list.at(index)->widget;
 }
 
@@ -1463,7 +1459,7 @@ void QSplitter::moveSplitter(int pos, int index)
 void QSplitter::getRange(int index, int *min, int *max) const
 {
     Q_D(const QSplitter);
-    d->getRange(index, min, 0, 0, max);
+    d->getRange(index, min, nullptr, nullptr, max);
 }
 
 
@@ -1486,7 +1482,8 @@ int QSplitter::closestLegalPosition(int pos, int index)
 
 /*!
     \property QSplitter::opaqueResize
-    \brief whether resizing is opaque
+    Returns \c true if widgets are resized dynamically (opaquely) while interactively moving the
+    splitter. Otherwise returns \c false.
 
     The default resize behavior is style dependent (determined by the
     SH_Splitter_OpaqueResize style hint). However, you can override it
